using System;
using System.Collections.Generic;
using System.Text;

namespace PXBVISACSharp
{
    
    public class Example
    {
        public static void Main(string[] args)
        {
            Example example = new Example();
            
            //Run example 1
            example.Example1();
            Console.Out.WriteLine("Press any key to run next example\n");
            Console.In.ReadLine();
            //Run example 2
            example.Example2();
            Console.Out.WriteLine("Press any key to run next example\n");
            Console.In.ReadLine();

            //Run example 3
            example.Example3();
            Console.Out.WriteLine("Press any key to run next example\n");
            Console.In.ReadLine();

            example.Example4();
            Console.Out.WriteLine("Press any key to run next example\n");
            Console.In.ReadLine();

            example.Example5();
            Console.Out.WriteLine("Press any key to exit\n");
            Console.In.ReadLine();
        }

        /// <summary>
        /// Users need to register external instruments to a PXB so that PXB can talk to the external instruments.
        /// To register an external instrument, users must provide the instrument name, address and instrument family.
        /// Currently, PXB only supports the following instrument family: ESG-C, MXG, MXA and DSIM.  PXB allows
        /// users to register a custom external instrument but PXB will not talk to that instrument.
        /// Please MODIFY THESE ADDRESSES to point to the appropriate instruments
        /// </summary>
        string m_pxbAddress = "TPCIP::999.999.999.999::INSTR";   //Address of PXB (MIMO Receiver Tester)
        string m_esg1Address = "TPCIP::888.888.888.888::INSTR";  //Address of first ESG (Signal Generator)
        string m_esg2Address = "TPCIP::777.777.777.777::INSTR";  //Address of second ESG (Signal Geneerator)
        string m_mxa1Address = "TPCIP::666.666.666.666::INSTR";  //Address of first MXA (Signal Analyzer)
        string m_mxa2Address = "TPCIP::555.555.555.555::INSTR";  //Address of second MXA (Signal Analyzer)
        string m_mxg1Address = "TPCIP::444.444.444.444::INSTR";  //Address of first MXG (Signal Generator)
        string m_mxg2Address = "TPCIP::333.333.333.333::INSTR";  //Address of second MXG (Signal Geneerator)

        //Varibles for instrument names
        //Please MODIFY these names to reflect your instrument names
        string m_esg1Name = "ESG1";  //Name of first ESG (Signal Generator)
        string m_esg2Name = "ESG2";  //Name of second ESG (Signal Geneerator)
        string m_mxa1Name = "MXA1";  //Name of first MXA (Signal Analyzer)
        string m_mxa2Name = "MXA2";  //Name of second MXA (Signal Analyzer)
        string m_mxg1Name = "MXG1";  //Name of first MXG (Signal Generator)
        string m_mxg2Name = "MXG2";  //Name of second MXG (Signal Geneerator)
        string m_dsim1Name = "DSIM1";
        string m_dsim2Name = "DSIM2";
        string m_customInst1Name = "Custom1";
        string m_customInst2Name = "Custom2";

        //Variables for supported instrument family
        const string s_esgFamily = "ESGC";
        const string s_mxgFamily = "MXG";
        const string s_mxaFamily = "MXA";
        const string s_dsimFamily = "DSIM";
        const string s_customFamily = "NONE";

        //Variables as reference to agilent instruments
        AgilentInstrument m_PXB;
        AgilentInstrument m_ESG1;
        AgilentInstrument m_ESG2;
        AgilentInstrument m_MXA1;
        AgilentInstrument m_MXA2;
        AgilentInstrument m_MXG1;
        AgilentInstrument m_MXG2;

        string m_freq = "1e-9";
        string m_power = "-20";
        const string s_on = "ON";
        const string s_off = "OFF";

        //Variables to store configure type and configuration name
        string m_configType = String.Empty;
        string m_configName = String.Empty;

        //Variables to store waveform file names
        string m_waveformFile1 = String.Empty;
        string m_waveformFile2 = String.Empty;

        public Example()
        {
            /********************************************/
            //STEP # 1:  Open connections to instruments
            /********************************************/
            //Open a connection to a PXB
            m_PXB = new AgilentInstrument();
            m_PXB.Connect(m_pxbAddress,180000);
            //Open connections to Signal Generators
            m_ESG1 = new AgilentInstrument();
            m_ESG1.Connect(m_esg1Address,180000);
            m_ESG2 = new AgilentInstrument();
            m_ESG2.Connect(m_esg2Address,180000);
            //Open connections to Signal Analyzers
            m_MXA1 = new AgilentInstrument();
            m_MXA1.Connect(m_mxa1Address,180000);
            m_MXA2 = new AgilentInstrument();
            m_MXA2.Connect(m_mxa2Address,180000);

            /********************************************************************************************/
            //STEP # 2:  Register external instruments to PXB using command "SYST:COMM:RDEV:ADD"
            //Users must register external instruments (Signal Generator, Signal Analyzer, DSIM or Custom Instrument) to PXB
            //before PXB can talk to those external instruments
            //PXB allows users to register up to 16 signal generators.  Currently, only ESG-C and MXG are supported
            //PXB allows users to register up to 8 signal analyzers.  Currently, only MXA is supported
            //PXB allows users to register up to 8 DSIM and up to 8 custom instruments
            /********************************************************************************************/
            //Register ESG1 and ESG2
            m_PXB.Write(String.Format("SYST:COMM:RDEV:ADD \"{0}\",{1},\"{2}\"", m_esg1Name, s_esgFamily, m_esg1Address));
            m_PXB.Write(String.Format("SYST:COMM:RDEV:ADD \"{0}\",{1},\"{2}\"", m_esg2Name, s_esgFamily, m_esg2Address));
            //Register MXG1 and MXG2
            m_PXB.Write(String.Format("SYST:COMM:RDEV:ADD \"{0}\",{1},\"{2}\"", m_mxg1Name, s_mxgFamily, m_mxg1Address));
            m_PXB.Write(String.Format("SYST:COMM:RDEV:ADD \"{0}\",{1},\"{2}\"", m_mxg2Name, s_mxgFamily, m_mxg2Address));
            //Register MXA1 and MXA2
            m_PXB.Write(String.Format("SYST:COMM:RDEV:ADD \"{0}\",{1},\"{2}\"", m_mxa1Name, s_mxaFamily, m_mxa1Address));
            m_PXB.Write(String.Format("SYST:COMM:RDEV:ADD \"{0}\",{1},\"{2}\"", m_mxa2Name, s_mxaFamily, m_mxa2Address));
            //Register DSIM1 and DSIM2.  Instrument address is not applicable for DSIM
            m_PXB.Write(String.Format("SYST:COMM:RDEV:ADD \"{0}\",{1},\"{2}\"", m_dsim1Name, s_dsimFamily, String.Empty));
            m_PXB.Write(String.Format("SYST:COMM:RDEV:ADD \"{0}\",{1},\"{2}\"", m_dsim2Name, s_dsimFamily, String.Empty));
            //Register custom instruments.  Instrument address is not applicable for custom instruments
            m_PXB.Write(String.Format("SYST:COMM:RDEV:ADD \"{0}\",{1},\"{2}\"", m_customInst1Name, s_customFamily, String.Empty));
            m_PXB.Write(String.Format("SYST:COMM:RDEV:ADD \"{0}\",{1},\"{2}\"", m_customInst2Name, s_customFamily, String.Empty));

            /********************************************************************************************/
            //STEP # 3:  Configure PXB
            //Users must SELECT a predefined configuration, CONNECT appropriate external instruments to
            //PXB's IO ports and finally LOAD that configuration.
            //A PXB's configuration is a unique hardware resource allocation with a unique signal routing.
            //In a PXB, a configuration is presented as a block diagram that includes some DSP blocks and some
            //I/O blocks.  The block diagram also indicates the signal flow from one block to other blocks. 
            //Valid configuration list varies depend on the number of Baseband cards, I/O cards and 
            //installed licenses.  Currently, a PXB can have up to 8 I/O blocks (or 4 I/O cards) and 12 DSP
            //blocks (or 6 Baseband cards).  A DSP block can be configured as a baseband generator block or 
            //a baseband simulator block (or Fading block).  An I/O block can be configured as an input block 
            //or an output block.
            //To view the full list of predefined configuration and its block diagram, please start up a PXB
            //in simulation mode.
            //
            //CONNECT external instruments to IO ports also has some restriction.  Currently, users can
            //only connect a signal generator (ESG-C or MXG), a DSIM or a custom instrument to an output port.  For input
            //port, only a signal analyzer (MXA) is allowed.
            //DSIM and MXA can only be connected to the I/O digital port.  Signal generator can be connected to
            //either an I/O digital port or an I/O analog port.  Custom instrument can only be connected to
            //an I/O analog port.  Make sure to use appropriate cable to connect external instruments to PXB's I/O ports
            //By default, I/O1 is mapped to port A1, I/O2 is mapped to port A2, 
            //            I/O3 is mapped to port B1, I/O4 is mapped to port B2,
            //            I/O5 is mapped to port C1, I/O6 is mapped to port C4,
            //            I/O7 is mapped to port D1, I/O8 is mapped to port D4
            //
            //Please view Example1, Example2 and Example3 for specific configuration
            /********************************************************************************************/
            
        }

        /// <summary>
        /// Configure PXB as One Channel Baseband Generator, download default waveform and play
        /// </summary>
        public void Example1()
        {
            ///////////////////////////////////////////////////////////////////////////////
            //Configuration #1: Streaming out signal from one DSP block to an output block
            //Configuration Type: GEN, Configuration Name: "1Ch"
            m_configType = "GEN";  //or GENerate
            m_configName = "1Ch";  //means 1 Channel
            //Select a configuration
            m_PXB.Write(String.Format("CONT:CONF {0},\"{1}\";*WAI\n", m_configType, m_configName));
            //Tell PXB that I/O1 (port A1) is connected to signal generator ESG1 via the digital port
            //DON'T FORGET to connect a DIGITAL CABLE between port A1 to the ESG1
            m_PXB.Write(String.Format("CONT:CONF:IO1:DIG:RDEV \"{0}\";*WAI\n", m_esg1Name));
            //Load the selected configuration
            m_PXB.Write("CONT:CONF:APPL;*WAI\n");


            /*******************************************************************************************
             * STEP #4: SELECT waveforms for baseband generator block and DOWNLOAD the waveforms
             * A baseband generator block is presented by "RAD" term.  For all configuration of type GEN,
             *     RAD1 will stream signal to output I/O1
             *     RAD2 will stream signal to output I/O2
             *     RAD3 will stream signal to output I/O3
             *     RAD4 will stream signal to output I/O4
             * Be default, a baseband generator block has a sine waveform file.  However, users can always
             * select a different waveform file.
             * *****************************************************************************************/
            //To select a different waveform file such as "C:\Waveforms\Waveform1.wfm", please replace the
            //String.Empty with "C:\Waveforms\Waveform1.wfm"
            m_waveformFile1 = String.Empty; //Please replace String.Emtpy with the real file name
            if (m_waveformFile1 != String.Empty)
            {
                //Select a different waveform for baseband generator block 1, for block 2, use RAD2, etc...
                m_PXB.Write(String.Format("RAD1:ARB:WAV {0}", m_waveformFile1));
            }
            //Download all waveforms.  This task may take a long time depend on the size of the waveforms
            m_PXB.Write("RAD:ARB:ALL:LOAD ON;*WAI\n");

            /*******************************************************************************************
             * STEP #5:  Setup signal generator.  In this example, we will setup the ESG1 as the following
             *   Frequency is set to 1 GHz
             *   Power is set to -20 dBm
             *   Turn on RF output
             * ****************************************************************************************/
            m_freq = "1e-9";
            m_power = "-20";
            m_ESG1.Write(String.Format(":FREQ {0}", m_freq));
            m_ESG1.Write(String.Format(":POW {0}", m_power));
            m_ESG1.Write(String.Format(":OUTP {0};*WAI\n", s_on));

            /*******************************************************************************************
             * STEP #6:  PLAY out the waveforms using different trigger source
             * 
             * By default, the internal software (SW) trigger source is used.  However, users can always set up
             * a different trigger source such as external hardware (HW) trigger source or BUS trigger source
             * *****************************************************************************************/

            m_PXB.Write(String.Format("CONT:PLAY {0}", s_on)); //Play out signal using SW trigger source
            System.Threading.Thread.Sleep(5000);
            //Up to now, if there is no error, users can verify that there is a RF output from
            //the ESG1.  Let's change PXB configuration to address another type of DSP block: Fading block
            m_PXB.Write(String.Format("CONT:PLAY {0}", s_off)); //Stop the signal to before swith to new configuration

        }

        /// <summary>
        /// Set up PXB as SISO configuration with 2 Fading channels.  
        /// Set up Fading channel 1 as ...
        /// Set up Fading channel 2 as ...
        /// </summary>
        public void Example2()
        {
            //----------------------------------------------------------------------------------------//
            //----------------------------------------------------------------------------------------//
            //////////////////////////////////////////////////////////////////////////////////////////
            //Configuration #2: Streaming out signal from a baseband generator block to a fading block
            //to simulate the signal and finally output the signal to an output block
            //Configuration Type: GFAD, Configuration Name: "2Ch"
            m_configType = "GFAD";  //or Generate and Fade
            m_configName = "2Ch";  //means 1 Channel
            //Select a configuration
            m_PXB.Write(String.Format("CONT:CONF {0},\"{1}\";*WAI\n", m_configType, m_configName));
            //Tell PXB that I/O1 (port A1) is connected to signal generator ESG1 via the digital port
            //DON'T FORGET to connect a DIGITAL CABLE between port A1 to the ESG1
            m_PXB.Write(String.Format("CONT:CONF:IO1:DIG:RDEV \"{0}\";*WAI\n", m_esg1Name));
            //Tell PXB that I/O2 (port A2) is connected to signal generator ESG2 via the digital port
            //DON'T FORGET to connect a DIGITAL CABLE between port A2 to the ESG2
            m_PXB.Write(String.Format("CONT:CONF:IO2:DIG:RDEV \"{0}\";*WAI\n", m_esg2Name));
            //Load the selected configuration
            m_PXB.Write("CONT:CONF:APPL;*WAI\n");
            //Select a waveform, for example a constant 1 waveform if you have one
            m_waveformFile1 = String.Empty; //Please replace String.Emtpy with the real file name
            m_waveformFile2 = String.Empty;
            if (m_waveformFile1 != String.Empty)
            {
                //Select a different waveform for baseband generator block 1, use RAD1
                m_PXB.Write(String.Format("RAD1:ARB:WAV {0}", m_waveformFile1));
            }
            if (m_waveformFile2 != String.Empty)
            {
                //Select a different waveform for baseband generator block 2, use RAD2
                m_PXB.Write(String.Format("RAD2:ARB:WAV {0}", m_waveformFile2));
            }
            //Download all waveforms.  This task may take a long time depend on the size of the waveforms
            m_PXB.Write("RAD:ARB:ALL:LOAD ON;*WAI\n");

            //********************************************************************************
            //Setup Fading:
            //  Set up Fading channel 1 as LTE, Uplink (Mobile Station), Extended Typical Urban, Band X
            //  Set up Fading channel 2 as Fixed WiMAX/SUI-6/Omni antenna 90% cell coverage
            //********************************************************************************
            m_technology = "LTE";
            m_scenario = "ETU";  //Extended Typical Urban
            m_linkDirection = "MOB";//mobile
            m_freqBand = "10";//Frequency Band
            m_PXB.Write(String.Format("FSIM1:FAD1:STAN:TECH {0}", m_technology));
            m_PXB.Write(String.Format("FSIM1:FAD1:STAN:SCEN {0}", m_scenario));
            m_PXB.Write)String,Format("FSIM1:FAD1:STAN:LINK {0}", m_linkDirection)); 
            m_PXB.Write)String,Format("FSIM1:FAD1:STAN:FBAN {0}", m_freqBand));         
            
            m_technology = "FWIM";
            m_scenario = "SUI6";
            m_linkDirection = "MOB"; //mobile
            m_freqBand = "10"; //Frequency Band
            m_PXB.Write(String.Format("FSIM1:FAD2:STAN:TECH {0}", m_technology));
            m_PXB.Write(String.Format("FSIM1:FAD2:STAN:SCEN {0}", m_scenario));
            m_PXB.Write(String.Format("FSIM1:FAD2:STAN:LINK {0}", m_linkDirection));
            m_PXB.Write(String.Format("FSIM1:FAD2:STAN:FBAN {0}", m_freqBand));

            //Setup Fading paths with Rayleigh fading 1.6 kHz Doppler frequency
            int numOfPaths = 24;  //A fading channel (or a fading block) has 24 paths
            for (int fader = 1; fader <= 2; fader++)
            {
                for (int path = 1; path <= numOfPaths; path++)
                {
                    m_PXB.Write(String.Format(":FSIM1:FAD{0}:PATH{1}:ENABle 1", fader.ToString(), path.ToString()));
                    m_PXB.Write(String.Format(":FSIM1:FAD{0}:PATH{1}:DELay 0", fader.ToString(), path.ToString()));
                    m_PXB.Write(String.Format(":FSIM1:FAD{0}:PATH{1}:LOSS 0", fader.ToString(), path.ToString()));
                    m_PXB.Write(String.Format(":FSIM1:FAD{0}:PATH{1}:FTYPe Rayleigh", fader.ToString(), path.ToString()));
                    m_PXB.Write(String.Format(":FSIM1:FAD{0}:PATH{1}:DFR 1600", fader.ToString(), path.ToString()));
                }
            }
            m_freq = "1e-9";
            m_power = "-20";
            //Turn on the ESG1 RF output
            m_ESG1.Write(String.Format(":FREQ {0}", m_freq));
            m_ESG1.Write(String.Format(":POW {0}", m_power));
            m_ESG1.Write(String.Format(":OUTP {0};*WAI\n", s_on));

            //Turn on the ESG1 RF output
            m_ESG2.Write(String.Format(":FREQ {0}", m_freq));
            m_ESG2.Write(String.Format(":POW {0}", m_power));
            m_ESG2.Write(String.Format(":OUTP {0};*WAI\n", s_on));

            //Play
            m_PXB.Write(String.Format("CONT:PLAY {0};*WAI\n", s_on));
            System.Threading.Thread.Sleep(5000);
            m_PXB.Write(String.Format("CONT:PLAY {0}", s_off)); //Stop the signal to before swith to new configuration

        }

        string m_carrierFrequency = String.Empty; //variable for fading carrier frequency
        string m_technology = String.Empty;  //variable for Fading technology
        string m_linkDirection = String.Empty; //variable for Fading link direction
        string m_scenario = String.Empty; //variable for Fading scenario
        string m_freqBand = String.Empty; //variable for Fading frequency band
        string m_dopplerFreq = String.Empty; //variable for Fading doppler frequency

        /// <summary>
        /// Set up PXB as single user 2x2 MIMO, set up Fading as LTE Mobile Extended Typical Urban High Doppler
        /// Frequency and Mobil Wimax Base Station Vehicle A High Speed
        /// </summary>
        public void Example3()
        {
            //--------------------------------------------------------------------------------------//
            //--------------------------------------------------------------------------------------//
            //////////////////////////////////////////////////////////////////////////////////////////
            //Configuration #3: Single user 2x2 MIMO.
            // 
            //Configuration Type: MIMO, Configuration Name: "2x2"
            m_configType = "MIMO";  //or Multiple inputs and Multiple outputs
            m_configName = "2x2";  //means 2 baseband generator blocks, 4 fading blocks and 2 output blocks
            //Select a configuration
            m_PXB.Write(String.Format("CONT:CONF {0},\"{1}\";*WAI\n", m_configType, m_configName));
            //Tell PXB that I/O1 (port A1) is connected to signal generator ESG1 via the digital port
            //DON'T FORGET to connect a DIGITAL CABLE between port A1 to the ESG1
            //Tell PXB that I/O2 (port A2) is connected to signal generator ESG2 via the digital port
            //DON'T FORGET to connect a DIGITAL CABLE between port A2 to the ESG2
            m_PXB.Write(String.Format("CONT:CONF:IO1:DIG:RDEV \"{0}\";*WAI\n", m_esg1Name));
            m_PXB.Write(String.Format("CONT:CONF:IO2:DIG:RDEV \"{0}\";*WAI\n", m_esg2Name));
            //Load the selected configuration
            m_PXB.Write("CONT:CONF:APPL;*WAI\n");
            //Select a waveform, for example a constant 1 waveform if you have one
            m_waveformFile1 = String.Empty; //Please replace String.Emtpy with the real file name
            string m_waveformFile2 = String.Empty;
            if (m_waveformFile1 != String.Empty)
            {
                //Select a different waveform for baseband generator block 1
                m_PXB.Write(String.Format("RAD1:ARB:WAV {0}", m_waveformFile1));
            }
            if (m_waveformFile2 != String.Empty)
            {
                //Select a different waveform for baseband generator block 2
                m_PXB.Write(String.Format("RAD2:ARB:WAV {0}", m_waveformFile2));
            }
            //Download all waveforms.  This task may take a long time depend on the size of the waveforms
            m_PXB.Write("RAD:ARB:ALL:LOAD ON;*WAI\n");

            //***************************************************************************************
            //Setup Fading:
            //  Set Carier Frequency as 1.2 GHz
            //  Select Standard LTE Technology with Moble Station Link Direction, Extended Typical Urban,
            //  High Doppler Frequency
            //***************************************************************************************
            m_carrierFrequency = "1.2e9"; //1.2GHz
            m_technology = "LTE";  //LTE
            m_linkDirection = "UP"; // will be MOBile in the future
            m_scenario = "ETU"; //Extended Typical Urban
            m_dopplerFreq = "HIGH"; //High Doppler Frequency
            m_PXB.Write(String.Format("FSIM1:FREQ {0}", m_carrierFrequency));
            m_PXB.Write(String.Format("FSIM1:STAN:TECH {0}",m_technology));
            m_PXB.Write(String.Format("FSIM1:STAN:LINK {0}",m_linkDirection));
            m_PXB.Write(String.Format("FSIM1:STAN:LTE:SCEN {0}",m_scenario));
            m_PXB.Write(String.Format("FSIM1:STAN:LTE:DFR {0}",m_dopplerFreq));

            //Turn the ESG1 RF output on
            m_ESG1.Write(String.Format(":FREQ {0}", m_freq));
            m_ESG1.Write(String.Format(":POW {0}", m_power));
            m_ESG1.Write(String.Format(":OUTP {0};*WAI\n", s_on));
            //Turn the ESG2 RF output on
            m_ESG2.Write(String.Format(":FREQ {0}", m_freq));
            m_ESG2.Write(String.Format(":POW {0}", m_power));
            m_ESG2.Write(String.Format(":OUTP {0};*WAI\n", s_on));
            //Play
            m_PXB.Write(String.Format("CONT:PLAY {0};*WAI\n", s_on));
            System.Threading.Thread.Sleep(5000);
            m_PXB.Write(String.Format("CONT:PLAY {0}", s_off)); //Stop

            //***************************************************************************************
            //Change Fading Setup to:
            //  Set Carier Frequency 2.4 GHz
            //  Select Mobile Wimax Standard Technology with Base Station Link Direction, Extended Typical Urban,
            //
            //***************************************************************************************
            m_carrierFrequency = "2.4e9";
            m_technology = "MWIM";  //Mobil Wimax
            m_linkDirection = "DOWN"; //Will be Base Station in the future
            m_scenario = "VAHS"; //Vehicular A, High Speed (120 km/h) profile
            m_PXB.Write(String.Format("FSIM1:FREQ {0}", m_carrierFrequency));
            m_PXB.Write(String.Format("FSIM1:STAN:TECH {0}", m_technology));
            m_PXB.Write(String.Format("FSIM1:STAN:LINK {0}", m_linkDirection));
            m_PXB.Write(String.Format("FSIM1:STAN:LTE:SCEN {0}", m_scenario));
            
            m_freq = "1e-9";
            m_power = "-20";
            //Turn the ESG1 RF output on
            m_ESG1.Write(String.Format(":FREQ {0}", m_freq));
            m_ESG1.Write(String.Format(":POW {0}", m_power));
            m_ESG1.Write(String.Format(":OUTP {0};*WAI\n", s_on));
            //Turn the ESG2 RF output on
            m_ESG2.Write(String.Format(":FREQ {0}", m_freq));
            m_ESG2.Write(String.Format(":POW {0}", m_power));
            m_ESG2.Write(String.Format(":OUTP {0};*WAI\n", s_on));
            //Play
            m_PXB.Write(String.Format("CONT:PLAY {0};*WAI\n", s_on));
            System.Threading.Thread.Sleep(5000);
            m_PXB.Write(String.Format("CONT:PLAY {0}", s_off)); //Stop

        }

        /// <summary>
        /// Set up PXB as Generate and Sum configuration with 2 Playback channels.  
        /// Set up marker generation, marker routing and output markers
        /// </summary>
        public void Example4()
        {
            //----------------------------------------------------------------------------------------//
            //----------------------------------------------------------------------------------------//
            //////////////////////////////////////////////////////////////////////////////////////////
            //Configuration #2: Streaming out signal from a baseband generator block to a fading block
            //to simulate the signal and finally output the signal to an output block
            //Configuration Type: GSUM, Configuration Name: "2Ch"
            m_configType = "GSUM";  //or Generate and Fade
            m_configName = "2Ch";  //means 1 Channel
            //Select a configuration
            m_PXB.Write(String.Format("CONT:CONF {0},\"{1}\";*WAI\n", m_configType, m_configName));
            //Tell PXB that I/O1 (port A1) is connected to signal generator ESG1 via the digital port
            //DON'T FORGET to connect a DIGITAL CABLE between port A1 to the ESG1
            m_PXB.Write(String.Format("CONT:CONF:IO1:DIG:RDEV \"{0}\";*WAI\n", m_esg1Name));
            
            //Load the selected configuration
            m_PXB.Write("CONT:CONF:APPL;*WAI\n");
            //Select a waveform, for example a constant 1 waveform if you have one
            m_waveformFile1 = String.Empty; //Please replace String.Emtpy with the real file name
            m_waveformFile2 = String.Empty;
            if (m_waveformFile1 != String.Empty)
            {
                //Select a different waveform for baseband generator block 1, use RAD1
                m_PXB.Write(String.Format("RAD1:ARB:WAV {0}", m_waveformFile1));
            }
            if (m_waveformFile2 != String.Empty)
            {
                //Select a different waveform for baseband generator block 2, use RAD2
                m_PXB.Write(String.Format("RAD2:ARB:WAV {0}", m_waveformFile2));
            }
            //Download all waveforms.  This task may take a long time depend on the size of the waveforms
            m_PXB.Write("RAD:ARB:ALL:LOAD ON;*WAI\n");

            //********************************************************************************
            //Setup Marker Generating:
            //  Set up Marker Generation for baseband generator block 1 to use a marker file source
            //  for all markers (Marker1, Marker3 and Marker4).  Note that Marker2 is for 
            //  internal use only.
            //         
            //  Set up Marker Generation for baseband generator block 2 as Embeded source where:
            //         Marker 1 is dynamic, periodic
            //         Marker 3 is dynamic, power range detect
            //         Marker 4 is embeded
            //********************************************************************************
            string m_markerSource = "FILE"; //file marker source
            //Set baseband block 1 marker source to be user file source
            m_PXB.Write(String.Format("RAD1:ARB:MSO {0}",m_markerSource));
            int markerIndex = 1;
            //Turn all the markers of block 1 ON
            for(markerIndex = 1; markerIndex < 4 && markerIndex != 2; markerIndex++)
            {
                m_PXB.Write(String.Format("RAD1:ARB:MARK{0}:ENAB {1}",markerIndex.ToString(),s_on));
            }
            //Set baseband block 2 marker source to be "Embeded"
            m_markerSource = "EMB"; //embeded marker source
            m_PXB.Write(String.Format("RAD2:ARB:MSO {0}", m_markerSource));
            //Turn all the markers of block 2 ON
            for (markerIndex = 1; markerIndex < 4 && markerIndex != 2; markerIndex++)
            {
                m_PXB.Write(String.Format("RAD2:ARB:MARK{0}:ENAB {1}", markerIndex.ToString(), s_on));
            }
            //Set marker 1 (of block 2) to dynamic, periodic with pulse start 2 samples, 
            //pulse width as 4 samples and pulse period as 8 samples
            m_PXB.Write("RAD2:ARB:MARK1:SOURce DYN");//dynamic
            m_PXB.Write("RAD2:ARB:MARK1:TYPE PER");  //periodic type
            m_PXB.Write("RAD2:ARB:MARK1:TYPE:PER:PST 2"); //pulse start as 2 samples
            m_PXB.Write("RAD2:ARB:MARK1:TYPE:PER:PWID 4"); //pulse witdth as 4 samples
            m_PXB.Write("RAD2:ARB:MARK1:TYPE:PER:PPER 8"); //pulse period as 8 samples

            //Set marker 3 (of block 2) to dynamic, range detect, relationship as Equal, data type as Power,
            //detect when power equal to -3 dB
            m_PXB.Write("RAD2:ARB:MARK3:SOURce DYN");//dynamic
            m_PXB.Write("RAD2:ARB:MARK3:TYPE RDET");  //range detect
            m_PXB.Write("RAD2:ARB:MARK3:TYPE:RREL EQU"); //relationship as Equal
            m_PXB.Write("RAD2:ARB:MARK3:TYPE:RREL:RDAT POW"); //data type as power
            m_PXB.Write("RAD2:ARB:MARK3:TYPE:RREL:UNIT DB"); //unit as dB
            m_PXB.Write("RAD2:ARB:MARK3:TYPE:RREL:EQU -3"); //compare to -3.0 dB

            //Set marker 4 (of block 2) as embeded, delay 5 ns
            m_PXB.Write("RAD2:ARB:MARK4:SOURce EMB");//embeded
            m_PXB.Write("RAD2:ARB:MARK4:DEL 5e-9");  //delay 5 ns

            //********************************************************************************
            //Setup Marker Routing and Output Markers:
            //  Set up Marker Routing: route markers from baseband block 2 to output block
            //         
            //  Set up Output Markers:
            //         Output Marker 1: keeps it the same as the one routed from baseband block 2.
            //                          Please see Marker 1 in Marker Generation section above
            //         Output Marker 3: override the one routed from baseband block 2.
            //                          Set it as DYN, range detect, Q data greater than 10%
            //         Output Marker 4: keeps it the same as the one routed from baseband block 2.
            //                          Please see Marker 4 in Marker Generation section above
            //********************************************************************************
            
            //Route markers from baseband block 2 to output using master channel command
            string m_masterChannel = "BB Gen2";
            m_PXB.Write(String.Format("CONT:IO1:OUTP:MCH \"{0}\"", m_masterChannel));
            //Set up output Marker 1 the same as the one routed from baseband block 2
            //By not changing anything

            //Override output Marker 3 routed from baseband block 2 by set it up
            //as dynamic, range detec, Q data greater 10%
            m_PXB.Write("RAD2:ARB:MARK3:SOURce DYN");//dynamic
            m_PXB.Write("RAD2:ARB:MARK3:TYPE RDET");  //range detect
            m_PXB.Write("RAD2:ARB:MARK3:TYPE:RREL GRE"); //relationship as Greater Than
            m_PXB.Write("RAD2:ARB:MARK3:TYPE:RREL:RDAT Q"); //data type as Q data
            m_PXB.Write("RAD2:ARB:MARK3:TYPE:RREL:UNIT PCT"); //unit as percent
            m_PXB.Write("RAD2:ARB:MARK3:TYPE:RREL:GRE 10"); //compare to 10%

            //Set up output Marker 4 the same as the one routed from baseband block 2
            //By not changing anything
           
            //Play and test the output markers and stop
            m_PXB.Write(String.Format("CONT:PLAY {0};*WAI\n", s_on));
            System.Threading.Thread.Sleep(5000);
            m_PXB.Write(String.Format("CONT:PLAY {0}", s_off)); //Stop the signal

            //Now reset output Marker 3 back to be the same as the one routed from baseband block 2
            //and test it.
            m_PXB.Write("RAD2:ARB:MARK3:SOURce MCH");//use the one routed from baseband block 2
            //Play and test the output markers and stop
            m_PXB.Write(String.Format("CONT:PLAY {0};*WAI\n", s_on));
            System.Threading.Thread.Sleep(5000);
            m_PXB.Write(String.Format("CONT:PLAY {0}", s_off)); //Stop the signal
        }

        /// <summary>
        /// Test AWGN
        /// Set up PXB as SISO configuration with 2 Fading channels.  
        /// Set up AWGN to test power control with different cases:
        ///     Case 1: Fading off, test AWGN with signal only, with noise only then with signal and noise
        ///     Case 2: Fading on, test AWGN with signal only, with noise only then with signal and noise
        /// 
        /// </summary>
        public void Example5()
        {
            //----------------------------------------------------------------------------------------//
            //----------------------------------------------------------------------------------------//
            //////////////////////////////////////////////////////////////////////////////////////////
            //Configuration #2: Streaming out signal from a baseband generator block to a fading block
            //to simulate the signal and finally output the signal to an output block
            //Configuration Type: GFAD, Configuration Name: "2Ch"
            m_configType = "GFAD";  //or Generate and Fade
            m_configName = "2Ch";  //means 1 Channel
            //Select a configuration
            m_PXB.Write(String.Format("CONT:CONF {0},\"{1}\";*WAI\n", m_configType, m_configName));
            //Tell PXB that I/O1 (port A1) is connected to signal generator ESG1 via the digital port
            //DON'T FORGET to connect a DIGITAL CABLE between port A1 to the ESG1
            m_PXB.Write(String.Format("CONT:CONF:IO1:DIG:RDEV \"{0}\";*WAI\n", m_esg1Name));
            //Tell PXB that I/O2 (port A2) is connected to signal generator ESG2 via the digital port
            //DON'T FORGET to connect a DIGITAL CABLE between port A2 to the ESG2
            m_PXB.Write(String.Format("CONT:CONF:IO2:DIG:RDEV \"{0}\";*WAI\n", m_esg2Name));
            //Load the selected configuration
            m_PXB.Write("CONT:CONF:APPL;*WAI\n");
            //Select a waveform, for example a constant 1 waveform if you have one
            m_waveformFile1 = String.Empty; //Please replace String.Emtpy with the real file name
            m_waveformFile2 = String.Empty;
            if (m_waveformFile1 != String.Empty)
            {
                //Select a different waveform for baseband generator block 1, use RAD1
                m_PXB.Write(String.Format("RAD1:ARB:WAV {0}", m_waveformFile1));
            }
            if (m_waveformFile2 != String.Empty)
            {
                //Select a different waveform for baseband generator block 2, use RAD2
                m_PXB.Write(String.Format("RAD2:ARB:WAV {0}", m_waveformFile2));
            }
            //Download all waveforms.  This task may take a long time depend on the size of the waveforms
            m_PXB.Write("RAD:ARB:ALL:LOAD ON;*WAI\n");

            //********************************************************************************
            //Setup Fading:
            //  Set up Fading channel 1 as LTE, Uplink (Mobile Station), Extended Typical Urban, Band X
            //  Set up Fading channel 2 as Fixed WiMAX/SUI-6/Omni antenna 90% cell coverage
            //********************************************************************************
            m_technology = "LTE";
            m_scenario = "ETU"; //Extended Typical Urban
            m_linkDirection = "MOB";//mobile
            m_freqBand = "10";//Frequency Band
            m_PXB.Write(String.Format("FSIM1:FAD1:STAN:TECH {0}", m_technology));
            m_PXB.Write(String.Format("FSIM1:FAD1:STAN:SCEN {0}", m_scenario));
            m_PXB.Write)String,Format("FSIM1:FAD1:STAN:LINK {0}", m_linkDirection)); 
            m_PXB.Write)String,Format("FSIM1:FAD1:STAN:FBAN {0}", m_freqBand)); 

            m_technology = "FWIM";
            m_scenario = "SUI6";
            m_linkDirection = "MOB"; //mobile
            m_freqBand = "10"; //Frequency Band
            m_PXB.Write(String.Format("FSIM1:FAD2:STAN:TECH {0}", m_technology));
            m_PXB.Write(String.Format("FSIM1:FAD2:STAN:SCEN {0}", m_scenario));
            m_PXB.Write(String.Format("FSIM1:FAD2:STAN:LINK {0}", m_linkDirection));
            m_PXB.Write(String.Format("FSIM1:FAD2:STAN:FBAN {0}", m_freqBand));

            //Setup Fading paths with Rayleigh fading 1.6 kHz Doppler frequency
            int numOfPaths = 24;  //A fading channel (or a fading block) has 24 paths
            for (int fader = 1; fader <= 2; fader++)
            {
                for (int path = 1; path <= numOfPaths; path++)
                {
                    m_PXB.Write(String.Format(":FSIM1:FAD{0}:PATH{1}:ENABle 1", fader.ToString(), path.ToString()));
                    m_PXB.Write(String.Format(":FSIM1:FAD{0}:PATH{1}:DELay 0", fader.ToString(), path.ToString()));
                    m_PXB.Write(String.Format(":FSIM1:FAD{0}:PATH{1}:LOSS 0", fader.ToString(), path.ToString()));
                    m_PXB.Write(String.Format(":FSIM1:FAD{0}:PATH{1}:FTYPe Rayleigh", fader.ToString(), path.ToString()));
                    m_PXB.Write(String.Format(":FSIM1:FAD{0}:PATH{1}:DFR 1600", fader.ToString(), path.ToString()));
                }
            }

            //Turn on AWGN, perform power calibration.  Assume all the signal generators already setup properly
            string m_awgnMux = "SUM";  //signal + noise
            m_PXB.Write("CONT:IO1:OUTP:AWGN:ENAB ON"); //turn on AWGN on output 1
            m_PXB.Write(String.Format("CONT:IO1:OUTP:AWGN:MUX {0}", m_awgnMux));
            m_PXB.Write("CONT:IO2:OUTP:AWGN:ENAB ON"); //turn on AWGN on output 2
            m_PXB.Write(String.Format("CONT:IO2:OUTP:AWGN:MUX {0}", m_awgnMux));
            m_PXB.Write("CAL:POW ON"); //Start power calibration
            m_PXB.Write(String.Format("CONT:PLAY {0};*WAI\n", s_on)); //Play out the signal

            /*************************************************************************************
             * Case 1: Fading off
             *      Test AWGN with signal only
             *      Test AWGN with noise only
             *      Test AWGN with signal and noise
             * ***********************************************************************************/
            m_PXB.Write(":FSIM1:FAD1:MODE DIS"); //Set mode as DISabled to turn off fading while playing
            m_PXB.Write(":FSIM1:FAD2:MODE DIS"); 
            m_awgnMux = "SIG"; //set MUX to signal only then observe the output signals
            m_PXB.Write(String.Format("CONT:IO1:OUTP:AWGN:MUX {0}", m_awgnMux));
            m_PXB.Write(String.Format("CONT:IO2:OUTP:AWGN:MUX {0}", m_awgnMux));
            System.Threading.Thread.Sleep(5000);
            //Note that MUX is run-time setting.  We don't have to stop the signal

            //Now, set MUX to noise only then observe the output signals
            m_awgnMux = "NOIS";
            m_PXB.Write(String.Format("CONT:IO1:OUTP:AWGN:MUX {0}", m_awgnMux));
            m_PXB.Write(String.Format("CONT:IO2:OUTP:AWGN:MUX {0}", m_awgnMux));
            System.Threading.Thread.Sleep(5000);
            //Now, set MUX back to signal and noise then observe the output signals
            m_awgnMux = "SUM";
            m_PXB.Write(String.Format("CONT:IO1:OUTP:AWGN:MUX {0}", m_awgnMux));
            m_PXB.Write(String.Format("CONT:IO2:OUTP:AWGN:MUX {0}", m_awgnMux));
            System.Threading.Thread.Sleep(5000);
            /*************************************************************************************
             * Case 2: Fading on
             *      Test AWGN with signal only
             *      Test AWGN with noise only
             *      Test AWGN with signal and noise
             * ***********************************************************************************/
            m_PXB.Write(":FSIM1:FAD1:MODE ENAB"); //Set mode as ENABled to turn on fading while playing
            m_PXB.Write(":FSIM1:FAD2:MODE ENAB");
            //Set MUX to signal only then observe the output signals
            m_awgnMux = "SIG"; 
            m_PXB.Write(String.Format("CONT:IO1:OUTP:AWGN:MUX {0}", m_awgnMux));
            m_PXB.Write(String.Format("CONT:IO2:OUTP:AWGN:MUX {0}", m_awgnMux));
            System.Threading.Thread.Sleep(5000);
            //Note that MUX is run-time setting.  We don't have to stop the signal

            //Now, set MUX to noise only then observe the output signals
            m_awgnMux = "NOIS";
            m_PXB.Write(String.Format("CONT:IO1:OUTP:AWGN:MUX {0}", m_awgnMux));
            m_PXB.Write(String.Format("CONT:IO2:OUTP:AWGN:MUX {0}", m_awgnMux));
            System.Threading.Thread.Sleep(5000);
            //Now, set MUX back to signal and noise then observe the output signals
            m_awgnMux = "SUM";
            m_PXB.Write(String.Format("CONT:IO1:OUTP:AWGN:MUX {0}", m_awgnMux));
            m_PXB.Write(String.Format("CONT:IO2:OUTP:AWGN:MUX {0}", m_awgnMux));
            System.Threading.Thread.Sleep(5000);
           
            m_PXB.Write(String.Format("CONT:PLAY {0}", s_off)); //Stop the signal to before swith to new configuration

        }
    }
}
